define {
ES_AUTOHSCROLL = 0x0080
ES_PASSWORD    = 0x0020

EN_CHANGE      = 0x0300

EM_GETLIMITTEXT     = 0x00D5
EM_LIMITTEXT        = 0x00C5
EM_SETSEL           = 0x00B1
EM_GETSEL           = 0x00B0
EM_SETPASSWORDCHAR  = 0x00CC
EM_GETPASSWORDCHAR  = 0x00D2
EM_SETREADONLY      = 0x00CF
}

type vedit <inherit = vctrl>
{  
   uint flgsetnewtext
   
   uint p_changed
   uint p_readonly
   
   onevent onchange
}

property uint vedit.readonly :return this.p_readonly

property  vedit.readonly( uint val )
{
   if val != this.p_readonly
   {
      this.p_readonly = 1
      this.winmsg( $EM_SETREADONLY, val )
   }
}

property uint vedit.changed
{
   return this.p_changed
}

property vedit.changed( uint val )
{
   this.p_changed = val
}

property str vedit.txt <result>
{
   uint res = GetWindowTextLength( this.hwnd ) + 1
   result.reserve( res )     
   GetWindowText( this.hwnd, result.ptr(), res )
   result.setlenptr()
}

property vedit.txt( str val )
{   
   this.flgsetnewtext = 1
   SetWindowText( this.hwnd, val.ptr() )
   this.flgsetnewtext = 0
}

property uint vedit.maxlen
{
   return this.winmsg( $EM_GETLIMITTEXT )
}

property vedit.maxlen( uint val )
{
   this.winmsg( $EM_LIMITTEXT, val )
}

method vedit.sel( uint start, uint len )
{
   this.winmsg( $EM_SETSEL, start, start + len ) 
} 

method vedit.selall()
{
   this.sel( 0, -1 )
}

property uint vedit.selstart
{   
   uint start
   this.winmsg( $EM_GETSEL, &start, 0 )
   return start  
}

property vedit.selstart( uint val )
{
   this.sel( val, 0 )
}

property uint vedit.sellen
{
   uint start, end   
   this.winmsg( $EM_GETSEL, &start, &end )
   return end - start   
}

property vedit.sellen( uint val )
{
   this.sel( this.selstart, val )    
}

property str vedit.seltxt<result>
{
   uint start, end
   str  txt   
   this.winmsg( $EM_GETSEL, &start, &end )
   result.substr(this.txt, start, end-start )
}

property uint vedit.password
{   
   return  ?( GetWindowLong( this.hwnd, $GWL_STYLE ) & $ES_PASSWORD, 1, 0 )
}

property vedit.password( uint val )
{
   if val
   {
      this.winmsg( $EM_SETPASSWORDCHAR, '*')
      this.txt = this.txt
   }
   else
   {
      this.winmsg( $EM_SETPASSWORDCHAR, 0 )
      this.txt = this.txt   
   }
}

property vedit.border( uint val )
{
   uint style = GetWindowLong( this.hwnd, $GWL_EXSTYLE )
   if val : style |= $WS_EX_CLIENTEDGE
   else : style &= ~$WS_EX_CLIENTEDGE
   SetWindowLong( this.hwnd, $GWL_EXSTYLE, style )      
   SetWindowPos( this.hwnd, 0, 0, 0, 0, 0, $SWP_FRAMECHANGED | 
                  $SWP_NOACTIVATE | $SWP_NOZORDER | $SWP_NOMOVE | $SWP_NOSIZE )
}

property uint vedit.border
{   
   return ?(GetWindowLong( this.hwnd, $GWL_EXSTYLE ) & $WS_EX_CLIENTEDGE,1,0)
}

method uint vedit.defproc( eventn ev )
{  
   if ev.code == $e_winntf && ev->eventwnd.message == $WM_COMMAND &&
      ev->eventwnd.wpar >> 16 == $EN_CHANGE  
   {  
      this.p_changed = !this.flgsetnewtext
      this.onchange.run()  
   }
   elif ev.code == $e_focus && ev->eventuint.val
   {
      this.selall()
   }
   return this->vctrl.defproc( ev )
}


method vedit vedit.init( )
{  
   this.p_typename = "vedit"   
   this.p_typeid = vedit
   this.setdefproc()
   this.p_tabstop = 1
   this.canfocus = 1
   this.loc.width = 100
   this.loc.height = 25       
   return this 
}  

method vedit vedit.win()
{   
   this.hwnd = createwin( $WS_EX_CLIENTEDGE, "EDIT", "", 
      $ES_AUTOHSCROLL | $WS_CHILD | $WS_VISIBLE | $WS_CLIPSIBLINGS, 
      0, 0, this.loc.width, this.loc.height, 
      this.p_owner->vctrl.hwnd, 0, 0, &this )
   this.winmsg( $WM_SETFONT, GetStockObject( $DEFAULT_GUI_FONT ))
   return this
}

//------------------------------------------------------------------------------

method vedit.getprops( uint typeid, compMan cm )
{
   this->vctrl.getprops( typeid, cm)
   cm.addprops( typeid, %{ 
"taborder"     , uint   , 0,
"txt"          , str    , 0,
"maxlen"       , uint   , 0,
"border"       , uint   , 1,
"password"     , uint   , 0
   })
}

method vedit.getevents( uint typeid, compMan cm )
{
   this->vctrl.getevents( typeid, cm)
   cm.addevents( typeid, %{ 
"onchange"     , "eventn"     ,
"onkey"        , "eventkey"   ,
"onfocus"      , "eventuint" 
   })
}